#!/usr/bin/perl

##########################################################################
#
#  fancydns.pl - September 22, 1999 - Graham Mainwaring <graham@mhn.org>
#
#  This is an example script intended to be called from pump when an
#  interface receives its DHCP configuration. In addition to doing the
#  normal stuff you would expect, this script updates a local named
#  instance and sets its forwarders to point to the DNS servers given
#  by DHCP, yet leaves /etc/resolv.conf pointing at the local server.
#  This might be useful if you are running a local caching nameserver,
#  yet want to take advantage of on-site DNS servers wherever you are.
#
#  Note that for this script to work, you must be running BIND 8 using
#  named.conf (not named.boot), and your named.conf must have an options
#  section that commences with a line containing just "options {", ends
#  with a line containing just "};", and the forwarders directive must
#  be all on one line like this: "forwarders { 1.2.3.4; 5.6.7.8; };".
#  
##########################################################################

use Sys::Syslog;

sub logerr($) {
  openlog('pumpd', 'cons,pid', 'daemon');
  syslog('err', $_[0]);
  closelog();
}

sub errexit($) {
  logerr($_[0]);
  die $_[0];
}

while (<>) {
  chop;
  ($tag,$value)=split(/\=/);
  $tag=lc($tag);
  if ($tag eq "dev") { $dev = $value }
  elsif ($tag eq "ip") { $ip = $value }
  elsif ($tag eq "netmask") { $netmask = $value }
  elsif ($tag eq "broadcast") { $broadcast = $value }
  elsif ($tag eq "network") { $network = $value }
  elsif ($tag eq "gateway") { $gateway = $value }
  elsif ($tag eq "hostname") { $hostname = $value }
  elsif ($tag eq "domain") { $domain = $value }
  elsif ($tag =~ /dns[0-9]/) { $dns[$#dns+1] = $value }
}

if (defined $dev && defined $ip && defined $netmask && defined $broadcast) {
  system "/sbin/ifconfig $dev $ip netmask $netmask broadcast $broadcast";
} else {
  errexit("did not get enough information to configure interface");
}

if (defined $gateway) {
  system "/sbin/route del default";
  system "/sbin/route add default gw $gateway";
} else {
  logerr("did not get a default gateway");
}

if (defined $hostname) {
  system "/bin/hostname $hostname";
}

open RESOLV, "/etc/resolv.conf";
while (<RESOLV>) {
  chop;
  ($key,$value)=split(/\s/);
  $key = lc($key);
  if ($key eq "nameserver") {
    $old_nameserver=$value;
  } elsif ($key eq "domain") {
    $old_domain=$value;
  } elsif ($key eq "search") {
    $old_search=$value;
  } else {
    $resolv[$#resolv+1]=$_;
  }
}
close RESOLV;

open RESOLV, ">/etc/resolv.conf";
if (defined $old_nameserver) {
  print RESOLV "nameserver $old_nameserver\n";
} elsif ($#dns >= 0) {
  print RESOLV "nameserver ";
  foreach $dns (@dns) {
    print RESOLV "$dns ";
  }
  print RESOLV "\n";
}
if (defined $domain) {
  print RESOLV "domain $domain\n";
} elsif (defined $old_domain) {
  print RESOLV "domain $old_domain\n";
}
if (defined $old_search) {
  print RESOLV "search $old_search\n";
}
foreach $resolv (@resolv) {
  print RESOLV "$resolv\n";
}
close RESOLV;

open NAMED, "/etc/named.conf";
while (<NAMED>) {
  chop;
  $named[$#named+1] = $_;
}
close NAMED;

sub write_forwarders ()
{
  $looking=0;
  print NAMED "	forwarders { ";
  foreach $dns (@dns) {
    print NAMED "$dns; ";
  }
  print NAMED "};\n";
}

$looking=0;
open NAMED, ">/etc/named.conf";
foreach $named (@named) {
  if ($looking == 1) {
    if (lc($named) =~ /forwarders/) {
      write_forwarders();
    } elsif ($named =~ /^\s*\}\;\s*$/) {
      write_forwarders();
      print NAMED "$named\n";
    } else {
      print NAMED "$named\n";
    }
  } else {
    if (lc($named) =~ /options \{/) {
      $looking=1;
    }
    print NAMED "$named\n";
  }
}
close NAMED;

system("/usr/bin/killall -HUP named");
